@@ -20,8 +20,8 @@
 #include <QPointer>
 #include <kconfig.h>
 #include <kdebug.h>
-#include <klocale.h>
-#include <kmenu.h>
+#include <KLocalizedString>
+
 #include <kactionmenu.h>
 #include <kmessagebox.h>
 #include <kicon.h>
@@ -127,7 +127,7 @@ ICQAccount::ICQAccount(Kopete::Protocol *parent, QString accountID)
 	QString nickName = configGroup()->readEntry("NickName", QString() );
 	mWebAware = configGroup()->readEntry( "WebAware", false );
 	mHideIP = configGroup()->readEntry( "HideIP", true );
-	mInfoWidget = 0L;
+	mInfoWidget = nullptr;
 
 	QObject::connect( engine(), SIGNAL(userReadsStatusMessage(QString)),
 	                  this, SLOT(userReadsStatusMessage(QString)) );
@@ -135,11 +135,11 @@ ICQAccount::ICQAccount(Kopete::Protocol *parent, QString accountID)
 	                  this, SLOT(slotGotAuthRequest(QString,QString)) );
 
 	// Create actions
-	mEditInfoAction = new KAction( KIcon("user-properties"), i18n( "Edit User Info..." ), this );
-	QObject::connect( mEditInfoAction, SIGNAL(triggered(bool)), this, SLOT(slotUserInfo()) );
+    mEditInfoAction = new QAction( QIcon::fromTheme("user-properties"), i18n( "Edit User Info..." ), this );
+	QObject::connect(mEditInfoAction, &QAction::triggered, this, &ICQAccount::slotUserInfo);
 
 	mActionInvisible = new KToggleAction( i18n( "In&visible" ), this );
-	QObject::connect( mActionInvisible, SIGNAL(triggered(bool)), this, SLOT(slotToggleInvisible()) );
+	QObject::connect(mActionInvisible, &KToggleAction::triggered, this, &ICQAccount::slotToggleInvisible);
 
 	//setIgnoreUnknownContacts(pluginData(protocol(), "IgnoreUnknownContacts").toUInt() == 1);
 
@@ -161,13 +161,11 @@ ICQProtocol* ICQAccount::protocol()
 	return static_cast<ICQProtocol*>(OscarAccount::protocol());
 }
 
-
 Oscar::Presence ICQAccount::presence()
 {
 	return protocol()->statusManager()->presenceOf( myself()->onlineStatus() );
 }
 
-
 void ICQAccount::fillActionMenu( KActionMenu *actionMenu )
 {
 	Kopete::Account::fillActionMenu( actionMenu );
@@ -179,7 +177,7 @@ void ICQAccount::fillActionMenu( KActionMenu *actionMenu )
 
 	Oscar::Presence pres( presence().type(), presence().flags() | Oscar::Presence::Invisible );
 	pres.setXtrazStatus( presence().xtrazStatus() );
-	mActionInvisible->setIcon( KIcon( protocol()->statusManager()->onlineStatusOf( pres ).iconFor( this ) ) );
+    mActionInvisible->setIcon( QIcon( protocol()->statusManager()->onlineStatusOf( pres ).iconFor( this ) ) );
 	mActionInvisible->setChecked( (presence().flags() & Oscar::Presence::Invisible) == Oscar::Presence::Invisible );
 	actionMenu->addAction( mActionInvisible );
 
@@ -190,12 +188,12 @@ void ICQAccount::fillActionMenu( KActionMenu *actionMenu )
 
 	KActionMenu *xtrazStatusMenu = new KActionMenu( i18n( "Set Xtraz Status" ), actionMenu );
 
-	KAction* xtrazStatusSetAction = new KAction( i18n( "Set Status..." ), xtrazStatusMenu );
-	QObject::connect( xtrazStatusSetAction, SIGNAL(triggered(bool)), this, SLOT(setXtrazStatus()) );
+	QAction* xtrazStatusSetAction = new QAction( i18n( "Set Status..." ), xtrazStatusMenu );
+	QObject::connect(xtrazStatusSetAction, &QAction::triggered, this, &ICQAccount::setXtrazStatus);
 	xtrazStatusMenu->addAction( xtrazStatusSetAction );
 
-	KAction* xtrazStatusEditAction = new KAction( i18n( "Edit Statuses..." ), xtrazStatusMenu );
-	QObject::connect( xtrazStatusEditAction, SIGNAL(triggered(bool)), this, SLOT(editXtrazStatuses()) );
+	QAction* xtrazStatusEditAction = new QAction( i18n( "Edit Statuses..." ), xtrazStatusMenu );
+	QObject::connect(xtrazStatusEditAction, &QAction::triggered, this, &ICQAccount::editXtrazStatuses);
 	xtrazStatusMenu->addAction( xtrazStatusEditAction );
 
 	ICQStatusManager* mgr = static_cast<ICQStatusManager *>(protocol()->statusManager());
@@ -207,15 +205,13 @@ void ICQAccount::fillActionMenu( KActionMenu *actionMenu )
 	for ( int i = 0; i < xtrazStatusList.count(); i++ )
 	{
 		Xtraz::StatusAction* xtrazAction = new Xtraz::StatusAction( xtrazStatusList.at(i), xtrazStatusMenu );
-		QObject::connect( xtrazAction, SIGNAL(triggered(Xtraz::Status)),
-		                  this, SLOT(setPresenceXStatus(Xtraz::Status)) );
+		QObject::connect(xtrazAction, &Xtraz::StatusAction::triggeredStatus, this, &ICQAccount::setPresenceXStatus);
 		xtrazStatusMenu->addAction( xtrazAction );
 	}
 
 	actionMenu->addAction( xtrazStatusMenu );
 }
 
-
 void ICQAccount::connectWithPassword( const QString &password )
 {
 	if ( password.isNull() )
@@ -296,7 +292,6 @@ void ICQAccount::disconnected( DisconnectReason reason )
 	OscarAccount::disconnected( reason );
 }
 
-
 void ICQAccount::slotToggleInvisible()
 {
 	using namespace Oscar;
@@ -318,8 +313,8 @@ void ICQAccount::slotUserInfo()
 			return;
 
 		mInfoWidget = new ICQUserInfoWidget( this, engine()->userId(), Kopete::UI::Global::mainWidget(), true );
-		QObject::connect( mInfoWidget, SIGNAL(finished()), this, SLOT(closeUserInfoDialog()) );
-		QObject::connect( mInfoWidget, SIGNAL(okClicked()), this, SLOT(storeUserInfoDialog()) );
+		QObject::connect(mInfoWidget, &ICQUserInfoWidget::finished, this, &ICQAccount::closeUserInfoDialog);
+		// QObject::connect(mInfoWidget, &ICQUserInfoWidget::okClicked, this, &ICQAccount::storeUserInfoDialog);
 		mInfoWidget->show();
 	}
 }
@@ -334,8 +329,8 @@ void ICQAccount::storeUserInfoDialog()
 void ICQAccount::closeUserInfoDialog()
 {
 	QObject::disconnect( this, 0, mInfoWidget, 0 );
-	mInfoWidget->delayedDestruct();
-	mInfoWidget = 0L;
+	mInfoWidget->deleteLater();
+	mInfoWidget = nullptr;
 }
 
 void ICQAccount::userReadsStatusMessage( const QString& contact )
@@ -491,7 +486,7 @@ void ICQAccount::slotGotAuthRequest( const QString& contact, const QString& reas
 	QString contactId = Oscar::normalize( contact );
 
 	Kopete::AddedInfoEvent* event = new Kopete::AddedInfoEvent( contactId, this );
-	QObject::connect( event, SIGNAL(actionActivated(uint)), this, SLOT(addedInfoEventActionActivated(uint)) );
+	QObject::connect(event, &Kopete::AddedInfoEvent::actionActivated, this, &ICQAccount::addedInfoEventActionActivated);
 
 	Kopete::AddedInfoEvent::ShowActionOptions actions = Kopete::AddedInfoEvent::AuthorizeAction;
 	actions |= Kopete::AddedInfoEvent::BlockAction;
@@ -536,8 +531,8 @@ void ICQAccount::addedInfoEventActionActivated( uint actionId )
 			else
 				info = new ICQUserInfoWidget( this, event->contactId(), Kopete::UI::Global::mainWidget() );
 
-			QObject::connect( info, SIGNAL(finished()), info, SLOT(delayedDestruct()) );
-			QObject::connect( event, SIGNAL(eventClosed(Kopete::InfoEvent*)), info, SLOT(delayedDestruct()) );
+			// QObject::connect(info, &ICQUserInfoWidget::finished, info, &ICQUserInfoWidget::delayedDestruct);
+			// QObject::connect(event, &Kopete::AddedInfoEvent::eventClosed, info, &ICQUserInfoWidget::delayedDestruct);
 			info->setModal( false );
 			info->show();
 		}
@@ -545,6 +540,3 @@ void ICQAccount::addedInfoEventActionActivated( uint actionId )
 	}
 }
 
-#include "icqaccount.moc"
-
-//kate: tab-width 4; indent-mode csands;
