@@ -28,11 +28,11 @@
 #include <kopetemetacontact.h>
 #include <kopetecontactaction.h>
 #include <qstring.h>
-#include <kaction.h>
+#include <QAction>
 #include <kactionmenu.h>
 #include <kactioncollection.h>
 #include <klocale.h>
-#include <kgenericfactory.h>
+#include <kpluginfactory.h>
 
 static Kopete::MetaContact *dummyContacts = new Kopete::MetaContact();
 
@@ -81,7 +81,7 @@ class SkypeChatSessionPrivate {
 			}
 		};
 		///The action to call the user(s)
-		KAction *callAction;
+		QAction *callAction;
 		///The action to invite the user
 		KActionMenu *inviteAction;
 		///The contact if any (and one)
@@ -110,9 +110,9 @@ SkypeChatSession::SkypeChatSession(SkypeAccount *account, SkypeContact *contact)
 	account->prepareChatSession(this);
 	d->isMulti = false;
 
-	d->callAction = new KAction(this);
+	d->callAction = new QAction(this);
 	d->callAction->setText(i18n("Call"));
-	d->callAction->setIcon(KIcon("voicecall"));
+	d->callAction->setIcon(QIcon::fromTheme(QStringLiteral("voicecall")));
 	connect(d->callAction, SIGNAL(triggered()), SLOT(callChatSession()));
 
 	connect(contact, SIGNAL(setActionsPossible(bool)), d->callAction, SLOT(setEnabled(bool)));
@@ -122,7 +122,7 @@ SkypeChatSession::SkypeChatSession(SkypeAccount *account, SkypeContact *contact)
 
 	d->contact = contact;
 
-	d->inviteAction = new KActionMenu (KIcon("system-users"), i18n ("&Invite"), this);
+	d->inviteAction = new KActionMenu (QIcon::fromTheme(QStringLiteral("system-users")), i18n ("&Invite"), this);
 	d->inviteAction->setDelayed(false);
 	connect( d->inviteAction->menu(), SIGNAL(aboutToShow()), this, SLOT(showInviteMenu()) );
 	connect( d->inviteAction->menu(), SIGNAL(aboutToHide()), this, SLOT(hideInviteMenu()) );
@@ -151,16 +151,16 @@ SkypeChatSession::SkypeChatSession(SkypeAccount *account, const QString &session
 	d->chatId = session;
 	emit updateChatId("", session, this);
 
-	d->callAction = new KAction(this);
+	d->callAction = new QAction(this);
 	d->callAction->setText(i18n("Call"));
-	d->callAction->setIcon(KIcon("voicecall"));
+	d->callAction->setIcon(QIcon::fromTheme(QStringLiteral("voicecall")));
 	connect(d->callAction, SIGNAL(triggered()), SLOT(callChatSession()));
 
 	actionCollection()->addAction("callSkypeContactFromChat", d->callAction);
 
 	disallowCall();//TODO I hope it will not be needed in future
 
-	d->inviteAction = new KActionMenu (KIcon("system-users"), i18n ("&Invite"), this);
+	d->inviteAction = new KActionMenu (QIcon::fromTheme(QStringLiteral("system-users")), i18n ("&Invite"), this);
 	d->inviteAction->setDelayed(false);
 	connect( d->inviteAction->menu(), SIGNAL(aboutToShow()), this, SLOT(showInviteMenu()) );
 	connect( d->inviteAction->menu(), SIGNAL(aboutToHide()), this, SLOT(hideInviteMenu()) );
@@ -292,7 +292,7 @@ void SkypeChatSession::showInviteMenu() {
 	QHash <QString, Kopete::Contact *> contactList = account()->contacts();
 	for ( QHash <QString, Kopete::Contact *>::Iterator it = contactList.begin(); it != contactList.end(); ++it ) {
 		if ( ! members().contains(it.value()) && it.value()->isOnline() && it.value()->onlineStatus().status() != Kopete::OnlineStatus::Offline ) {
-			KAction *a = new Kopete::UI::ContactAction(it.value(), actionCollection());
+			QAction *a = new Kopete::UI::ContactAction(it.value(), actionCollection());
 			connect( a, SIGNAL(triggered(QString,bool)), this, SLOT(inviteContact(QString)) );
 			d->inviteAction->addAction(a);
 		}
@@ -311,4 +311,3 @@ void SkypeChatSession::hideInviteMenu() {
 	d->inviteAction->menu()->clear();
 }
 
-#include "skypechatsession.moc"
